#pragma TextEncoding	= "UTF-8"
#pragma rtGlobals 		= 3
#pragma IgorVersion		= 6.3
#pragma ModuleName		= BackgroundRemover
#pragma version			= 1.31
#pragma DefaultTab		= {3,20,4}			// Set default tab width in Igor Pro 9 and later

// --------------------- Project Updater header ----------------------
// If you're using Igor Pro 8 or later and have Project Updater
// installed, this package can check periodically for new releases.
// https://www.wavemetrics.com/project/Updater
static Constant kProjectID = 21532 																// the project node on IgorExchange
static StrConstant ksShortTitle = "Background Remover"											// the project short title on IgorExchange

//________________________________________________________________________________________________
//	Written by Stephan Thuermer - https://www.wavemetrics.com/user/chozo
//	Can be used to remove the "background" from spectra using various models.
//	
//	Constant:		A constant value.
//	Step:			Smooth step function which connects the two endpoints (using tanh()).
//	Linear:			Linear function through the two endpoints.
//	Exponential:	Normalized exponential function connecting the two endpoints.
//	Polynomial:		Polynomial function fitted within the selected range.
//	Total Sum:		Total sum inelastic scattering background after X. Li et al., J.Electron Spectrosc. Relat. Phenom. 63 (1993) 253-265
//	Shirley:		Iterative Shirley inelastic scattering background after A. Proctor and P. A. Sherwood, Anal. Chem. 54, 13-19 (1982)
//	Tougaard:		Three- and four-parameter universal scattering background after S. Tougaard, Surf. Interface Anal. 25: 137-154 (1997)
//	Tougaard Data:	Scattering background using scattering cross-section data after S. Tougaard, Surf. Interface Anal. 25: 137-154 (1997)
//________________________________________________________________________________________________

//________________________________________________________________________________________________
// 2021-01-23 - ver. 1.30: initial public release
// 2021-01-25 - ver. 1.31: Fixed error when DataBroswer is not open & minor control size adjustments.
//________________________________________________________________________________________________

StrConstant	kBackgroundDir	 = "Background_Remover"																		// the name for the working dir
StrConstant	kBackgroundTitle = "Background Remover (ver 1.31 - 01/2021): Background of "								// string for the title bar of the main window
StrConstant	kBackgroundTypes = "Constant;Step;Linear;Exponential;Polynomial;Total Sum;Shirley;Tougaard;Tougaard Data;"	// the various types of backgrounds used (need to match with BackRem_ModeSelect() and BackRem_BackCalc())

Static StrConstant kTougaardIniPara = "2866;1643;0;0;"											// initial parameters B,C,D and T0 for the universal Tougaard background (common metal)
//Static StrConstant kTougaardIniPara = "325;542;275;5;"										// initial parameters B,C,D and T0 for SiO2
Static Constant kTougaardSmoothStep = 0.5														// how much the insulator step for parameter T0 will be smoothed (should not be 0)
Static Constant kMaxIterations		= 50														// iteration limit for Shirley background calculation
Static Constant kConvergelimit		= 1e-6														// converge limit for Shirley background calculation

Menu "Spectra Tools"
	"Background Subtraction ...",/Q, CreateBrowser; BackgroundSubtractGUI($GetBrowserSelection(0))
End

//################################################################################################

Structure BackRemoveStruct
	String FuncType
	Variable avgPercent	// y Values around range ends positions get averaged
	
	Variable expScale	// Exponential
	Variable polyDegree	// Poly
	Variable fullLinear	// Line
	// for Step
	Variable stepPos	// x center position
	Variable stepSmooth // smoothing value
	// for Tougaard (universal CS parameters)
	Variable TouB
	Variable TouC
	Variable TouD
	Variable TouT
	// Tougaard Data
	Variable CSscale	// scaling (default 1)
	String CSdata		// name of CS wave in current folder

	Variable followData	// follow data outside range
	Variable smoothData	// integer between 3 and 20
EndStructure

//################################################################################################

Function/S GenerateBackground(s, inwave, backwave, [from, to])
	STRUCT BackRemoveStruct &s
	Wave, inwave, backwave
	Variable from, to
	
	if (!WaveExists(inwave) && !WaveExists(backwave))
		return "Input waves missing."
	endif
	if (DimSize(inwave,0) != DimSize(backwave,0))
		return "Input waves need to have the same size and scaling."
	endif
	
	Variable rows	= DimSize(inwave,0)
	Variable delta	= DimDelta(inwave,0)
	from = ParamIsDefault(from) ? pnt2x(inwave,0) : from
	to	 = ParamIsDefault(to)	? pnt2x(inwave,rows-1) : to

	Variable xRange	= s.avgPercent/100*abs(rightx(inwave)-leftx(inwave))						// define the averaging range
	Variable Up		= mean(inwave, from-xRange, from+xRange)									// upper y value to connect to
	Variable Down	= mean(inwave, to-xRange, to+xRange)										// lower y value
	Variable P1		= x2pnt(inwave,from)
	Variable P2		= x2pnt(inwave,to)
	
	if (P1 < 0 || P2 > (rows-1))
		return "Range input is outsize data x scaling."
	endif
	
	backwave[,P1]	= Up																		// upper constant part of background	
	backwave[P2,]	= Down																		// lower constant part of background
	
	String message = ""
	Variable i = 0
	Strswitch(s.FuncType)
		case "Constant":																		// constant background just removes the baseline
			backwave = Down
		break
		case "Step":
			backwave = Down + (Up-Down)/2*(1-tanh(sign(delta)*(x-s.stepPos)/s.stepSmooth))		// tanh used to generate a smooth transition
		break
		case "Linear":
			if (s.fullLinear)
				backwave[0, rows-1] = Up - (Up-Down)*(p-P1)/(P2-P1)
			else
				backwave[P1, P2]	 = Up - (Up-Down)*(p-P1)/(P2-P1)
			endif
		break
		case "Exponential":
			backwave = s.expScale*(Up-Down)*(exp((P2-p)/(P2))-1)/(exp((P2-P1)/(P2))-1) + Down	// normalized exponential (is 1 at P1 and 0 at P2) which gets shifted and scaled
		break
		case "Polynomial":
			DFREF saveDFR = GetDataFolderDFR()
			SetDataFolder root:Packages:$(kBackgroundDir)
				Duplicate/Free inwave, FitWave, Mask
				Mask = 1; Mask[P1,P2] = 0														// the mask wave decides the fit range
				CurveFit/X=1/Q/L=(DimSize(inwave,0)) poly s.polyDegree, FitWave /M=Mask 
				Wave result = W_coef
				backwave = poly(result,x)														// write result
				KillWaves/Z W_Sigma, W_coef, W_ParamConfidenceInterval
			SetDataFolder saveDFR
		break
		case "Total Sum":
			Duplicate/Free inwave, temp; temp -= Down											// clean the data from the constant background first
			backwave[P1,P2] = Down+(Up-Down)*sum(temp,pnt2x(backwave,p),to)/sum(temp,from,to)	// Total background calculation
		break
		case "Shirley":
			Duplicate/Free backwave PrevShirley, Difference;	PrevShirley = Down
			do
				Difference = inwave - PrevShirley												// data minus constant background and previous Shirley iteration
				MatrixOP/O Difference = replaceNaNs(Difference,0)								// make sure there are no NaNs which disturb the sum later
				Variable KFactor = abs(Up-Down) / area(Difference,from,to)						// Shirley scale factor
				backwave[P1,P2] = KFactor*area(Difference,pnt2x(backwave,p),to)	+ Down			// Shirley background calculation
				MatrixOP/free Rest = abs(sum(PrevShirley - backwave))							// difference between the backgrounds before and after the current iteration
				i += 1
				if (i >= kMaxIterations)
					message +=  "Shirley calculation not converged. "
					backwave = NaN
					break
				endif
				PrevShirley = backwave															// save for next iteration
			while (Rest[0] > kConvergelimit*Up && i < kMaxIterations)
			message += "Iterations: "+ num2str(i)
		break
		case "Tougaard":
			DFREF inDir = GetWavesDataFolderDFR(inwave)
			Duplicate/O inwave, inDir:$(NameOfWave(inwave)+"_CS")								// prepare CS data wave
			Wave CS_Data = inDir:$(NameOfWave(inwave)+"_CS")
			SetScale/P x 0, abs(delta), CS_Data;	CS_Data = 0									// cross section data needs to be properly scaled from zero
			
			Variable Toutype = s.TouD > 0 ? -1 : 1												// see if parameter D is set; if yes the formula has to be changed in the denominator to minus
			CS_Data = (s.TouB * x) / ((s.TouC + Toutype * x^2)^2 + s.TouD * x^2)				// universal loss function
			CS_Data *= kTougaardSmoothStep*(1+tanh((x-s.TouT)/kTougaardSmoothStep))				// include band-gap parameter for insulators
			
			Duplicate/Free backwave Scatter;	Scatter = 0										// the scattering share per point
			backwave = 0																		// start with blank background
			for (i = 0; i < rows; i += 1)
				Scatter[0,rows-i-1] = CS_Data[p] * abs(delta) * (inwave[p+i]-Down)				// calculate the scattering from the current point on
				backwave[i] = sum(Scatter)														// sum up the current share
			endfor
			backwave += Down																	// add lower value
		break
		case "Tougaard Data":																	// no break here
			Duplicate/Free inwave, CS_Data														// prepare CS data wave
			SetScale/P x 0, abs(delta), CS_Data;	CS_Data = 0									// cross section data needs to be properly scaled from zero
			if (strlen(s.CSdata))
				Wave/Z CSWave = $s.CSdata
				if (WaveExists(CSWave))															// check if the wave and the scaling parameter are set
					CS_Data = 0
					P1 = ceil ((leftx(CSWave)-leftx(CS_Data))/deltax(CS_Data))
					P2 = trunc((leftx(CSWave)-leftx(CS_Data)+deltax(CSWave)*(numpnts(CSWave)-1))/deltax(CS_Data))		// boundary check
					P2 = P2 > rows-1 ? rows-1 : P2
					if (P1 > P2)
						message +=  "No useful CS data found."
						backwave = NaN
						break
					endif
					P1 = P1 < 0 ? 0 : P1														// protect against negative starting values
					CS_Data[P1,P2] = s.CSscale * CSWave(x)
					if (P2 < rows - 1)
						message +=  "CS data range too short."
					endif
				else
					backwave = NaN
					break
				endif
			endif
			
			Duplicate/Free backwave Scatter;	Scatter = 0										// the scattering share per point
			backwave = 0																		// start with blank background
			for (i = 0; i < rows; i += 1)
				Scatter[0,rows-i-1] = CS_Data[p] * abs(delta) * (inwave[p+i]-Down)				// calculate the scattering from the current point on
				backwave[i] = sum(Scatter)														// sum up the current share
			endfor
			backwave += Down																	// add lower value
		break
	Endswitch
	
	if (s.followData)
		Duplicate/Free inwave, temp
		if (s.smoothData > 0)
			Smooth limit(s.smoothData, 3, 20), temp
		endif
		backwave[0,P1]		= temp[p]
		backwave[P2,rows-1] = temp[p]
	endif
	return message
End

//################################################################################################

Function BackgroundSubtractGUI(inwave)
	Wave inwave

	if (!WaveExists(inwave))
		return -1
	endif
	if (WaveType(inwave,1) != 1)
		Abort "This is meant for numeric waves!"
	endif
	if (WaveDims(inwave) > 1)																	// make sure to only load 1D waves
		Abort "This is meant for 1D waves!"
	endif
	
	if (WinType("InelasticBackDisplay"))														// make sure that there is only one panel of this
		DoWindow/F InelasticBackDisplay
		DoAlert/T="Open Background Remover window found." 1, "There is another instance open. Close the old one and load the new data? Any changes will not be saved!"
		if (V_flag != 1)
			return -1
		endif
		KillWindow InelasticBackDisplay
	endif
	//+++++++++++++++++++++++++++++++++ generate working variables ++++++++++++++++++++++++++++++
	NewDataFolder/O	root:Packages
	NewDataFolder/O	root:Packages:$(kBackgroundDir)												// the folder for temporary stuff
	DFREF strg = root:Packages:$(kBackgroundDir)
	String/G strg:InwaveLocation = GetWavesDataFolder(inwave,2)									// save wave name and folder for later
	Duplicate/O inwave strg:Data_Orig, strg:Data_Net, strg:Data_Back, strg:NegResiduals			// make copies of the original wave for all needed stuff and work with these
	Wave/SDFR=strg Data_Orig, Data_Net, Data_Back, NegResiduals, CS_Data
	Data_Net = NaN; Data_Back = NaN; NegResiduals = NaN
	//+++++++++++++++++++++++++++++++++++ generate the panels +++++++++++++++++++++++++++++++++++
	Display/W=(230,50,875,470)/K=2/N=InelasticBackDisplay	Data_Orig as kBackgroundTitle + NameOfWave(inwave)
	AppendToGraph/C=(0,0,65280) 							Data_Back							// the calculated background
	AppendToGraph/C=(0,39168,0) 							Data_Net							// the spectrum with subtracted background
	AppendToGraph/C=(0,0,0)/L=Lresid 						NegResiduals						// negative values of the net spectrum recalculated for display
	
	ModifyGraph mirror(left)=1, mirror(bottom)=2, standoff=0, ZisZ=1, notation(left)=1, axisOnTop=1, zero(left)=4	// make the graph nice
	ModifyGraph axisEnab(left)={0.12,1},axisEnab(Lresid)={0,0.08}													// modify axis size
	ModifyGraph freePos(Lresid)=0,mode(NegResiduals) = 1, mirror(Lresid)=1,	lblPosMode(Lresid)=1					// bar display for residuals
	SetAxis/A/E=1 Lresid
	SetAxis/A/N=1 left																			// round counts to nice values
	//++++++++++++++++++++++++++++++++++++++++ controls +++++++++++++++++++++++++++++++++++++++++	
	ControlBar 60																				// add controls
	
	Button BMinus				,pos={10,5}		,size={65,23}		,title="Subtract"			,help={"Remove current background from the data."}
	Button BUndo				,pos={10,30}	,size={65,23}		,title="Undo"				,help={"Revert to the original data."}
	Button BSave				,pos={80,5}		,size={120,23}		,title="Save Background"	,help={"Save the current background (blue) and residuals (green)."}
	Button BStop				,pos={80,30}	,size={120,23}		,title="Overwrite & Quit"	,help={"Overwrites the original data and close the window."}
	PopUpMenu BackSelect		,pos={210,6}	,size={200,20}		,title="Type:"				,help={"Select the type of background to fit."}
	SetVariable ValA			,pos={210,35}	,size={95,23}		,title="Left:"				,help={"Set the left cursor position."}
	SetVariable ValB			,pos={315,35}	,size={95,23}		,title="Right:"				,help={"Set the right cursor position."}
	
	GroupBox Divider1			,pos={417,4}	,size={0,50}
	//+++++++++++++++++++++++++++++++ individual background controls ++++++++++++++++++++++++++++	
	SetVariable averageCurloc	,pos={475,8}	,size={95,23}		,title="Fuzzy Range:"		,help={"Percentage of total x range is used to take the average intensity as start and end."}
	SetVariable VarStepPos		,pos={475,35}	,size={95,23}		,title="Step Position:"		,help={"The position of the background step."}
	SetVariable VarStepSmooth	,pos={625,35}	,size={95,23}		,title="Smoothness:"		,help={"The smoothness of the background step."}
	SetVariable VarSmoothData	,pos={625,33}	,size={95,23}		,title="Smooth:"			,help={"The smoothness of the data outside the cursor range."}
	
	SetVariable VarPolyDegree	,pos={475,35}	,size={95,23}		,title="Poly Degree:"		,help={"Set the degree of the polynomial fit."}
	
	CheckBox VarFullLinear		,pos={432,35}	,size={143,23}		,title="Linear Over Full Range:"	,help={"Calculate the linear background for the full spectrum width."}
	CheckBox VarFollowData		,pos={432,35}	,size={168,23}		,title="Follow Data Outside Range:"	,help={"Makes the background identical with the smoothed original data outside the selected range."}
	
	SetVariable VarTougaardB	,pos={410,35}	,size={95,23}		,title="B:"					,help={"Parameter B (~ height) of the Tougaard universal CS."}
	SetVariable VarTougaardC	,pos={500,35}	,size={95,23}		,title="C:"					,help={"Parameter C (~ position of the maximum) of the Tougaard universal CS."}
	SetVariable VarTougaardD	,pos={590,35}	,size={95,23}		,title="D:"					,help={"Parameter D used for semi-conductors."}
	SetVariable VarTougaardT	,pos={680,35}	,size={95,23}		,title="T\B0\M:"			,help={"Parameter T0 (band-gap size) used additionally for insulators."}
	
	SetVariable VarCSScale		,pos={475,35}	,size={95,23}		,title="CS Scaling:"		,help={"Scales the cross section data to match the spectrum."}
	SetVariable VarExpScale		,pos={475,35}	,size={95,23}		,title="Scaling:"			,help={"Scales the exponential function to match the spectrum."}
	PopUpMenu VarCSData			,pos={625,33 }	,size={140,20}		,title="CS Data:"			,help={"Select a wave with correct cross section data for the material."}
	//+++++++++++++++++++++++++++++++++++ control modifiers ++++++++++++++++++++++++++++++++++++		
	Variable flip	= sign(DimDelta(Data_Orig,0)) == -1
	Variable left	= flip ? pnt2x(Data_Orig,numpnts(Data_Orig)-1) : Dimoffset(Data_Orig,0)		// parameters for cursor placement
	Variable right	= flip ? Dimoffset(Data_Orig,0) : pnt2x(Data_Orig,numpnts(Data_Orig)-1)		// for negative delta waves invert everything
	Variable inset	= (right-left)/100															// 1% of the wave range
	String unit		= WaveUnits(Data_Orig,0)
	WaveStats/Q/R = (left+10*inset,right-10*inset) Data_Orig
	Variable startStep = V_maxloc																// initial step position
	
	ModifyControlList ControlNameList("InelasticBackDisplay",";","B*")								,proc=BackRem_ButtonAction
	PopUpMenu BackSelect		,bodywidth=170	,value=kBackgroundTypes								,proc=BackRem_ModeSelect
	PopUpMenu VarCSData			,bodywidth=140	,value=BackRem_CSWaveList()							,proc=BackRem_CSWaveSelect
	SetVariable ValA			,bodywidth=65	,limits={left,right,inset}	,format="%g " + unit	,proc=BackRem_VarChange		,disable=2*(1-flip)
	SetVariable ValB			,bodywidth=65	,limits={left,right,inset}	,format="%g " + unit	,proc=BackRem_VarChange		,disable=2*(flip)
	SetVariable averageCurloc	,bodywidth=65	,limits={0,50,0.1}			,format="%g %"			,proc=BackRem_VarChange		,value=_NUM:0.5
	SetVariable VarStepPos		,bodywidth=65	,limits={0.1,50,0.1}		,format="%g " + unit	,proc=BackRem_VarChange		,value=_NUM:startStep
	SetVariable VarStepSmooth	,bodywidth=65	,limits={0.01,20,0.05}		,format="%g"			,proc=BackRem_VarChange		,value=_NUM:1
	SetVariable VarSmoothData	,bodywidth=65	,limits={0,500,1}			,format="%g"			,proc=BackRem_VarChange		,value=_NUM:5
	SetVariable VarTougaardB	,bodywidth=65	,limits={1,inf,10}			,format="%g"			,proc=BackRem_VarChange		,value=_NUM:str2num(StringFromList(0,kTougaardIniPara))
	SetVariable VarTougaardC	,bodywidth=65	,limits={1,inf,10}			,format="%g"			,proc=BackRem_VarChange		,value=_NUM:str2num(StringFromList(1,kTougaardIniPara))
	SetVariable VarTougaardD	,bodywidth=65	,limits={0,inf,10}			,format="%g"			,proc=BackRem_VarChange		,value=_NUM:str2num(StringFromList(2,kTougaardIniPara))
	SetVariable VarTougaardT	,bodywidth=65	,limits={0,100,1}			,format="%g"			,proc=BackRem_VarChange		,value=_NUM:str2num(StringFromList(3,kTougaardIniPara))
	SetVariable VarCSScale		,bodywidth=65	,limits={0.05,inf,0.05}		,format="%g"			,proc=BackRem_VarChange		,value=_NUM:1
	SetVariable VarExpScale		,bodywidth=65	,limits={0.05,inf,0.05}		,format="%g"			,proc=BackRem_VarChange		,value=_NUM:1
	SetVariable VarPolyDegree	,bodywidth=65	,limits={2,19,1}			,format="%g"			,proc=BackRem_VarChange		,value=_NUM:2
	CheckBox VarFullLinear													,side=1					,proc=BackRem_CheckAction
	CheckBox VarFollowData													,side=1					,proc=BackRem_CheckAction
	ModifyControlList ControlNameList("InelasticBackDisplay",";","Var*")	,disable=1			// hide all controls special controls
	DoUpdate
	
	Cursor/F/H=2/N=1/W=InelasticBackDisplay A  Data_Orig(left+10*inset),  Data_Orig(left+10*inset)	// set the first cursor on the graph
	Cursor/F/H=2/N=1/W=InelasticBackDisplay B  Data_Orig(right-10*inset), Data_Orig(right-10*inset)	// set the second cursor on the graph
	SetWindow InelasticBackDisplay hook(BackCalcAction)=BackRem_WindowHook							// link the function call to the window (after the cursors has been set; otherwise eventCode == 7 will trigger)

	BackRem_BackCalc()																			// initial background calculation
	DoWindow/F InelasticBackDisplay
	return 0
End
//################################################################################################
Function BackRem_WindowHook(s)																	// main window hook
	STRUCT WMWinHookStruct &s
	Variable HookTakeover = 0
	
	DFREF stg = root:Packages:$(kBackgroundDir)
	DFREF pkg = root:Packages:
	Switch (s.EventCode)
		case 7:																					// cursor position was modified
			BackRem_BackCalc()
		break
		case 8:		// graph was modified (probably axis zoom)
			HookTakeover = 1
			Wave/SDFR=stg Data_Orig
			ControlInfo/W=InelasticBackDisplay ValA
			Variable Left = V_Value
			ControlInfo/W=InelasticBackDisplay ValB
			Variable Right = V_Value
			Cursor/F/W=InelasticBackDisplay A  Data_Orig Left,  Data_Orig(Left)					// reset the cursors to their previous x value
			Cursor/F/W=InelasticBackDisplay B  Data_Orig Right, Data_Orig(Right)
		break
		case 17:	// window close button
			HookTakeover = 1
			KillWindow InelasticBackDisplay
			KillDataFolder stg
			Variable pkgObjects = CountObjectsDFR(pkg, 1)+CountObjectsDFR(pkg, 2)+CountObjectsDFR(pkg, 3)+CountObjectsDFR(pkg, 4)
			if (pkgObjects == 0)																// clean up Packages folder if empty
				KillDataFolder pkg
			endif
		break
	EndSwitch
	
	return HookTakeover
End
//################################################################################################
Function BackRem_ModeSelect(s) : PopupMenuControl
	STRUCT WMPopupAction &s
	if (s.eventCode != 2)
		return 0
	endif
	
	Variable onlyOneCursor = 0
	String list = ""
	Strswitch(s.popStr)
		case "Step":
			list = "VarStepPos;VarStepSmooth;"
		break
		case "Linear":
			list = "VarFullLinear;"
		break
		case "Exponential":
			list = "VarExpScale;"
		break
		case "Polynomial":
			list = "VarPolyDegree;"
		break
		case "Shirley":
		case "Total Sum":
			list = "VarFollowData;VarSmoothData;"
		break
		case "Constant":
			onlyOneCursor = 1
		break
		case "Tougaard":
			onlyOneCursor = 1
			list = ControlNameList("InelasticBackDisplay",";","VarTougaard*")
		break
		case "Tougaard Data":
			onlyOneCursor = 1
			list = "VarCSScale;VarCSData;"
		break
	Endswitch
	
	list += "ValA;ValB;"
	ModifyControlList ControlNameList("InelasticBackDisplay",";","Var*") ,win=InelasticBackDisplay ,disable=1	// hide all controls
	ModifyControlList list ,win=InelasticBackDisplay ,disable=0													// enable relevant controls
	
	if (onlyOneCursor)
		DFREF stg = root:Packages:$(kBackgroundDir)
		Wave/SDFR=stg Data_Orig
		if (sign(DimDelta(Data_Orig,0)) == -1)
			SetVariable ValB ,win=InelasticBackDisplay	,disable=2								// disable unused cursor (cosmetic change)
		else
			SetVariable ValA ,win=InelasticBackDisplay	,disable=2
		endif
	endif
	
	BackRem_BackCalc()
	return 0
End
//################################################################################################
Function BackRem_CheckAction (s) : CheckBoxControl
	STRUCT WMCheckboxAction &s
	if (s.eventCode == 2)
		BackRem_BackCalc()
	endif
	return 0
End
//################################################################################################
Function BackRem_CSWaveSelect(s) : PopupMenuControl
	STRUCT WMPopupAction &s
	if (s.eventCode == 2)
		BackRem_BackCalc()
	endif
	return 0
End
//+++++++++++++++++++++++++++++++++++++ popup help functions +++++++++++++++++++++++++++++++++++++
Function/S BackRem_CSWaveList()																	// create wave list for the drop down menu
	String list = "none;" + WaveList("*",";", "DIMS:1")
	return list
End
//################################################################################################
Function BackRem_ButtonAction(s) : ButtonControl
	STRUCT WMButtonAction &s
	if (s.eventCode != 2)
		return 0
	endif
	
	DFREF stg = root:Packages:$(kBackgroundDir)
	Wave/SDFR=stg Data_Orig, Data_Back, Data_Net
	SVAR inwaveLoc = stg:InwaveLocation
	Wave inwave = $inwaveLoc
	
	Variable mainCur, subCur;	GetCursorLoc(mainCur,subCur)
	ControlInfo/W=InelasticBackDisplay BackSelect
	
	String NewNote = "Background was removed!\r", AddNote = ""
	AddNote += "start = " + num2str(mainCur)  + " " + WaveUnits(inwave,0)
	AddNote += "\rend = " + num2str(subCur) + " " + WaveUnits(inwave,0)
	AddNote += "\rfunction = "	+ S_Value
	
	Strswitch(s.ctrlName)
		case "BMinus":
			Data_Orig -= Data_Back
		break
		case "BUndo":
			Duplicate/O inwave stg:Data_Orig
		break
		case "BSave":
			Duplicate/O stg:Data_Net  $(NameOfWave(inwave)+"_net")
			Duplicate/O stg:Data_Back $(NameOfWave(inwave)+"_bck")
			Note	$(NameOfWave(inwave)+"_net"), NewNote+AddNote
			Note/K	$(NameOfWave(inwave)+"_bck"), AddNote
		break
		case "BStop":
			if (!EqualWaves(Data_Orig, inwave,-1))		// compare if the wave was changed
				Duplicate/O stg:Data_Orig inwave
				Note inwave, NewNote+AddNote
			endif
			KillWindow InelasticBackDisplay
			return 0									// stop here
		break
	Endswitch
	
	BackRem_BackCalc()
	return 0
End
//################################################################################################
Function BackRem_VarChange(s) : SetVariableControl
	STRUCT WMSetVariableAction &s
	
	if (s.eventCode == 1 || s.eventCode == 2)
		DFREF storage = root:Packages:$(kBackgroundDir)
		Wave/SDFR=storage Data_Orig
		String CsrName = ""
		Strswitch(s.ctrlName)
			case "ValA":
				CsrName = SelectString(hcsr(A,"InelasticBackDisplay") > hcsr(B,"InelasticBackDisplay") , "A", "B")
			break
			case "ValB":
				CsrName = SelectString(hcsr(A,"InelasticBackDisplay") > hcsr(B,"InelasticBackDisplay") , "B", "A")
			break
			default:
				BackRem_BackCalc()
		EndSwitch
		If (strlen(CsrName))
			Cursor/F/W=InelasticBackDisplay $CsrName  Data_Orig s.dval, Data_Orig(s.dval)
		endif
	endif
	
	return 0
End
//################################################################################################
Function BackRem_ReadAllPanelSettings(info, low, high)											// read all GUI controls
	STRUCT BackRemoveStruct &info
	Variable low, high
	
	SetVariable VarStepPos	,win=InelasticBackDisplay ,limits={low,high,abs(high-low)*0.01}		// set limits of step control
	SetVariable ValA 		,win=InelasticBackDisplay ,value=_NUM:low
	SetVariable ValB 		,win=InelasticBackDisplay ,value=_NUM:high
	
	ControlInfo/W=InelasticBackDisplay BackSelect
	info.FuncType = S_Value
	ControlInfo/W=InelasticBackDisplay averageCurloc
	info.avgPercent = V_Value
	ControlInfo/W=InelasticBackDisplay VarSmoothData
	info.smoothData = V_Value
	ControlInfo/W=InelasticBackDisplay VarStepPos
	info.stepPos = limit(V_Value, low, high)													// apply limits to the parameter
	ControlInfo/W=InelasticBackDisplay VarStepSmooth
	info.stepSmooth = V_Value
	ControlInfo/W=InelasticBackDisplay VarTougaardB
	info.TouB = V_Value
	ControlInfo/W=InelasticBackDisplay VarTougaardC
	info.TouC = V_Value
	ControlInfo/W=InelasticBackDisplay VarTougaardD
	info.TouD = V_Value
	ControlInfo/W=InelasticBackDisplay VarTougaardT
	info.TouT = V_Value
	ControlInfo/W=InelasticBackDisplay VarCSScale
	info.CSscale = V_Value
	ControlInfo/W=InelasticBackDisplay VarExpScale
	info.expScale = V_Value
	ControlInfo/W=InelasticBackDisplay VarPolyDegree
	info.polyDegree = limit(V_Value+1,3,20)														// limit the poly degree between the possible 3 and 20
	ControlInfo/W=InelasticBackDisplay VarCSData												// get the wave to use for Tougaard calculation
	info.CSdata = SelectString(V_Disable == 0, "", S_Value)
	ControlInfo/W=InelasticBackDisplay VarFollowData
	info.followData = V_Value && V_Disable == 0													// visible and active
	ControlInfo/W=InelasticBackDisplay VarFullLinear
	info.fullLinear= V_Value
	return 0
End

//################################################################################################
Static Function BackRem_BackCalc()
	Variable mainCur, subCur;	GetCursorLoc(mainCur,subCur)
	Variable low  = mainCur < subCur ? mainCur : subCur
	Variable high = mainCur > subCur ? mainCur : subCur
	
	STRUCT BackRemoveStruct s
	BackRem_ReadAllPanelSettings(s, low, high)

	DFREF storage = root:Packages:$(kBackgroundDir)
	Wave/SDFR=storage Data_Orig, Data_Net, Data_Back, NegResiduals
	
	String GraphText = "\Z10\\s(Data_Orig) current spectrum"
	GraphText += "\r\\s(Data_Back) background"
	GraphText += "\r\\s(Data_Net) net spectrum"
	GraphText += "\r\\s(NegResiduals) negative values (%)"
	GraphText += "\K(65280,0,0)\f01"															// add color to the following text
	
	GraphText += "\r" + GenerateBackground(s, Data_Orig, Data_Back, from=mainCur, to=subCur)	// background calculation generates status messages
	
	Data_Net = Data_Orig - Data_Back															// subtract background
	NegResiduals = (Data_Net < 0) ? abs(Data_Net*100/Data_Orig) : NaN							// calculate residuals below zero
	Legend/C/N=GraphInfoBox/J/F=0/A=LT GraphText												// update legend box
	return 0
End
//################################################################################################
Static Function GetCursorLoc(mainCur,subCur)													// updates the current cursor location and gets the position
	Variable &mainCur, &subCur
	DFREF storage = root:Packages:$(kBackgroundDir)
	Wave/SDFR=storage Data_Orig
	
	mainCur	= hcsr(A,"InelasticBackDisplay")
	subCur	= hcsr(B,"InelasticBackDisplay")
	Variable flip = sign(DimDelta(Data_Orig,0))
	if (flip*mainCur > flip*subCur)
		mainCur	= hcsr(B,"InelasticBackDisplay")
		subCur	= hcsr(A,"InelasticBackDisplay")
	endif
	Variable lbound  = DimOffset(Data_Orig,0)
	Variable hbound = lbound + (DimSize(Data_Orig,0)-1)*DimDelta(Data_Orig,0)
	mainCur	= flip*mainCur < flip*lbound ? lbound : mainCur
	subCur	= flip*subCur  > flip*hbound ? hbound : subCur
	return 0
End